%let db2ssid = db2ssid;  /*Change DB2 Subsystem ID*/

/*----------------------------------------------------------------
  Program : Soooo_Sloooowly.sas
  Comments: Generate SAS data equivalent to DB2 tables.
  ----------------------------------------------------------------
  *1*: change 'c' to 'a' in %ex2.
  *2*: change msas and csas to m and c in %ex3.
  ----------------------------------------------------------------
            A = 517,000 rows
            B = 120,000 rows
            T1 = 20,000 rows
            T2 = 98,000 rows
SELECT *
  FROM   work.t1sas
        ,work.bsas
  WHERE  t1sas.account=bsas.account
    AND  t1sas.time=bsas.time
    AND  bsas.trans IN ('0123','4567','8901')
    AND  bsas.code_cf = 'C'
  ORDER BY
         t1sas.account
        ,t1sas.number
  ----------------------------------------------------------------
            M = 5,200,000 rows
            M' = 575,000 rows
            C = 640,000 rows
  SELECT COUNT(*)
  FROM   (SELECT *
          FROM   work.msas
          WHERE  SUBSTR(code,1,1) = 'A') AS msas1
  LEFT JOIN
         work.csas
  ON     msas1.account=csas.account
    AND  msas1.time=csas.time
  ----------------------------------------------------------------
            AX = 80,000 rows
            XC = 52,000 rows
            AX where in XC (column) = 4,300 rows
            AX inner join XC (column) = 10,595,900 rows
            AX distinct inner join XC (column) = 4,300 rows
            AX where not in XC (column) = 75,600 rows
            AX distinct left join XC (column) = 75,600 rows
  --------------------------------------------------------------*/

options fullstimer;

*-----------------------------------------------*;

*Summarising Rows in a DB2 Table*;

%macro ex1;
%let ex=ex1;

%put &ex. Step 1;
PROC SQL;
  CONNECT TO db2 (SSID=&db2ssid.);
  SELECT COUNT(*)
  FROM   CONNECTION TO db2
         (SELECT *
          FROM   a);
  DISCONNECT FROM db2;
QUIT;

%put &ex. Step 2a;
PROC ACCESS DBMS=db2;
  CREATE work.asas1.access;
  SSID=&db2ssid.;
  TABLE=a;
  ASSIGN=yes;
  CREATE work.asas.view;
  SELECT ALL;
RUN;

%put &ex. Step 2b;
PROC SQL;
  SELECT COUNT(*)
  FROM   work.asas;
QUIT;

LIBNAME db2sas1 DB2 SSID=&db2ssid.;

%put &ex. Step 3;
PROC SQL;
  SELECT COUNT(*)
  FROM   db2sas1.a;
QUIT;

%put &ex. Step 4;
PROC SQL;
  CONNECT TO db2 (SSID=&db2ssid.);
  SELECT *
  FROM   CONNECTION TO db2
         (SELECT COUNT(*)
          FROM   a);
  DISCONNECT FROM db2;
QUIT;
%mend ex1;

%ex1;

*-----------------------------------------------*;

*Combining Subsets of 2 DB2 Tables*;

%macro ex2;
%let ex=ex2;

%put &ex. Step 1a;
PROC ACCESS DBMS=db2;
  CREATE work.t1sas1.access;
  SSID=&db2ssid.;
  TABLE=t1;
  ASSIGN=yes;
  CREATE work.t1sas.view;
  SELECT ALL;
RUN;

%put &ex. Step 1b;
PROC ACCESS DBMS=db2;
  CREATE work.asas1.access;  /*1*/
  SSID=&db2ssid.;
  TABLE=a;  /*1*/
  ASSIGN=yes;
  CREATE work.asas.view;  /*1*/
  SELECT ALL;
RUN;

%put &ex. Step 1c;
PROC SQL;
  SELECT *
  FROM   work.t1sas
        ,work.asas  /*1*/
  WHERE  t1sas.account=asas.account  /*1*/
    AND  t1sas.time=asas.time  /*1*/
    AND  asas.trans IN ('0123','4567','8901')  /*1*/
    AND  asas.code_cf = 'C'  /*1*/
  ORDER BY
         t1sas.account
        ,t1sas.number;
QUIT;

LIBNAME db2sas2 DB2 SSID=&db2ssid.;

%put &ex. Step 2;
PROC SQL;
  SELECT t1.*
  FROM   db2sas2.t1
        ,db2sas2.a  /*1*/
  WHERE  t1.account=a.account  /*1*/
    AND  t1.time=a.time  /*1*/
    AND  a.trans IN ('0123','4567','8901')  /*1*/
    AND  a.code_cf = 'C'  /*1*/
  ORDER BY
         t1.account
        ,t1.number;
QUIT;

%put &ex. Step 3;
PROC SQL;
  CONNECT TO db2 (SSID=&db2ssid.);
  SELECT *
  FROM   CONNECTION TO db2
         (SELECT t1.*
                ,a.time AS atime  /*1*/
          FROM   t1
                ,a  /*1*/
          WHERE  t1.account=a.account  /*1*/
            AND  t1.time=a.time  /*1*/
            AND  a.trans IN ('0123','4567','8901')  /*1*/
            AND  a.code_cf = 'C') AS d  /*1*/
  ORDER BY
         d.account
        ,d.number;
  DISCONNECT FROM db2;
QUIT;

%put &ex. Step 4;
PROC SQL;
  CONNECT TO db2 (SSID=&db2ssid.);
  SELECT *
  FROM   CONNECTION TO db2
         (SELECT t1.*
                ,a.time AS atime  /*1*/
          FROM   t1
                ,a  /*1*/
          WHERE  t1.account=a.account  /*1*/
            AND  t1.time=a.time) AS d  /*1*/
  WHERE  d.trans IN ('0123','4567','8901')
    AND  d.code_cf = 'C'
  ORDER BY
         d.account
        ,d.number;
  DISCONNECT FROM db2;
QUIT;
%mend ex2;

%ex2;

*-----------------------------------------------*;

*Adding Extra Information to a DB2 Table from another DB2 Table;

%macro ex3;
%let ex=ex3;  /*2*/

%put &ex. Step 1a;
PROC ACCESS DBMS=db2;
  CREATE work.msas1.access;
  SSID=&db2ssid.;
  TABLE=m;
  ASSIGN=yes;
  CREATE work.msas.view;
  SELECT ALL;
RUN;

%put &ex. Step 1b;
PROC ACCESS DBMS=db2;
  CREATE work.csas1.access;
  SSID=&db2ssid.;
  TABLE=t1;
  ASSIGN=yes;
  CREATE work.csas.view;
  SELECT ALL;
RUN;

%put &ex. Step 1c;
PROC SQL;
  SELECT COUNT(*)
  FROM   (SELECT *
          FROM   work.msas
          WHERE  SUBSTR(code,1,1) = 'A') as msas1
  LEFT JOIN
         work.csas
  ON     msas1.account=csas.account
    AND  msas1.time=csas.time;
QUIT;

LIBNAME db2sas3 DB2 SSID=&db2ssid.;

%put &ex. Step 2;
PROC SQL;
  SELECT COUNT(*)
  FROM   db2sas3.m msas1  /*2*/
  LEFT JOIN
         db2sas3.c csas  /*2*/
  ON     msas1.account=csas.account
    AND  msas1.time=csas.time
  WHERE  SUBSTR(code,1,1) = 'A';
QUIT;

%put &ex. Step 3;
PROC SQL;
  CONNECT TO db2 (SSID=&db2ssid.);
  CREATE VIEW work.msas AS
    SELECT *
    FROM   CONNECTION TO db2
           (SELECT *
            FROM   m
            WHERE  SUBSTR(code,1,1) = 'A');
  CREATE VIEW work.csas AS
    SELECT *
    FROM   CONNECTION TO db2
           (SELECT *
            FROM   c);
  SELECT COUNT(*)
  FROM   (SELECT *
          FROM   work.msas) AS msas
  LEFT JOIN
         work.csas
  ON     msas.account=csas.account
    AND  msas.time=csas.time;
  DISCONNECT FROM db2;
QUIT;

%put &ex. Step 4;
PROC SQL;
  CONNECT TO db2 (SSID=&db2ssid.);
  SELECT COUNT(*)
  FROM   CONNECTION TO db2
         (SELECT *
          FROM   (SELECT *
                  FROM   m
                  WHERE  SUBSTR(code,1,1) = 'A') AS m1
          LEFT JOIN
                 c
          ON     m1.account=c.account
            AND  m1.time=c.time);
  DISCONNECT FROM db2;
QUIT;

%put &ex. Step 5;
PROC SQL;
  CONNECT TO db2 (SSID=&db2ssid.);
  SELECT *
  FROM   CONNECTION TO db2
         (SELECT COUNT(*)
          FROM   (SELECT *
                  FROM   m
                  WHERE  SUBSTR(code,1,1) = 'A') AS m1
          LEFT JOIN
                 c
          ON     m1.account=c.account
            AND  m1.time=c.time);
  DISCONNECT FROM db2;
QUIT;

%put &ex. Step 6;
PROC SQL;
  CONNECT TO db2 (SSID=&db2ssid.);
  SELECT COUNT(*)
  FROM   CONNECTION TO db2
         (SELECT *
          FROM   m
          LEFT JOIN
                 c
          ON     m.account=c.account
            AND  m.time=c.time)
  WHERE  SUBSTR(code,1,1) = 'A';
  DISCONNECT FROM db2;
QUIT;
%mend ex3;

%ex3;
