/*----------------------------------------------------------------
  Program : generate_data.sas
  Comments: Generate SAS data equivalent to DB2 tables.
  ----------------------------------------------------------------
            A = 517,000 rows
            B = 120,000 rows
            T1 = 20,000 rows
            T2 = 98,000 rows
SELECT *
  FROM   work.t1sas
        ,work.bsas
  WHERE  t1sas.account=bsas.account
    AND  t1sas.time=bsas.time
    AND  bsas.trans IN ('0123','4567','8901')
    AND  bsas.code_cf = 'C'
  ORDER BY
         t1sas.account
        ,t1sas.number
  ----------------------------------------------------------------
            M = 5,200,000 rows
            M' = 575,000 rows
            C = 640,000 rows
  SELECT COUNT(*)
  FROM   (SELECT *
          FROM   work.msas
          WHERE  SUBSTR(code,1,1) = 'A') AS msas1
  LEFT JOIN
         work.csas
  ON     msas1.account=csas.account
    AND  msas1.time=csas.time
  ----------------------------------------------------------------
            AX = 80,000 rows
            XC = 52,000 rows
            AX where in XC (column) = 4,300 rows
            AX inner join XC (column) = 10,595,900 rows
            AX distinct inner join XC (column) = 4,300 rows
            AX where not in XC (column) = 75,600 rows
            AX distinct left join XC (column) = 75,600 rows
  --------------------------------------------------------------*/

%let seed = 12345;

data a (drop = i);
  length account $10
         time 8
         trans $4
         code_cf $1
         column1 column2 column3 8
         ;
  i = ranuni(&seed);
  do i = 1 to 517000;
    account = put(ranuni(0) * 1000, z10.);
    time = choosen(int(ranuni(0) * 4) + 1, '09:00't, '12:00't, '15:00't, '18:00't, '21:00't);
    trans = choosec(int(ranuni(0) * 5) + 1, '0123', '4567', '8901', '2345', '6789', '0123');
    code_cf = choosec(int(ranuni(0) * 2) + 1, 'C', 'N', 'P');
    column1 = _n_;
    column2 = _n_;
    column3 = _n_;
    output;
  end;
run;

data b (drop = i);
  length account $10
         time 8
         trans $4
         code_cf $1
         column1 column2 column3 8
         ;
  i = ranuni(&seed);
  do i=1 to 120000;
    account = put(ranuni(0) * 1000, z10.);
    time = choosen(int(ranuni(0) * 4) + 1, '09:00't, '12:00't, '15:00't, '18:00't, '21:00't);
    trans = choosec(int(ranuni(0) * 5) + 1, '0123', '4567', '8901', '2345', '6789', '0123');
    code_cf = choosec(int(ranuni(0) * 2) + 1, 'C', 'N', 'P');
    column1 = _n_;
    column2 = _n_;
    column3 = _n_;
    output;
  end;
run;

data t1 (drop = i);
  length account $10
         time 8
         number 8
         column4 column5 column6 8
         ;
  i = ranuni(&seed);
  do i=1 to 20000;
    account = put(rancau(0) * 1000, z10.);
    time = choosen(int(ranuni(0) * 4) + 1, '09:00't, '12:00't, '15:00't, '18:00't, '21:00't);
    trans = choosec(int(ranuni(0) * 5) + 1, '0123', '4567', '8901', '2345', '6789', '0123');
    code_cf = choosec(int(ranuni(0) * 2) + 1, 'C', 'N', 'P');
    number = _n_;
    column4 = _n_;
    column5 = _n_;
    column6 = _n_;
    output;
  end;
run;

data t2 (drop = i);
  length account $10
         time 8
         number 8
         column4 column5 column6 8
         ;
  i = ranuni(&seed);
  do i=1 to 98000;
    account = put(rancau(0) * 1000, z10.);
    time = choosen(int(ranuni(0) * 4) + 1, '09:00't, '12:00't, '15:00't, '18:00't, '21:00't);
    code_cf = choosec(int(ranuni(0) * 2) + 1, 'C', 'N', 'P');
    number = _n_;
    column4 = _n_;
    column5 = _n_;
    column6 = _n_;
    output;
  end;
run;

data m (drop = i);
  length account $10
         time 8
         code $1
         column1 column2 column3 8
         ;
  i = ranuni(&seed);
  do i=1 to 5200000;
    account = put(ranuni(0) * 1000, z10.);
    time = choosen(int(ranuni(0) * 4) + 1, '09:00't, '12:00't, '15:00't, '18:00't, '21:00't);
    if (5200000 * ranuni(0)) le 575000 then code = 'A';
                                       else code = 'X';
    column1 = _n_;
    column2 = _n_;
    column3 = _n_;
    output;
  end;
run;

data c (drop = i);
  length account $10
         time 8
         column4 column5 column6 8
         ;
  i = ranuni(&seed);
  do i=1 to 640000;
    account = put(rancau(0) * 1000, z10.);
    time = choosen(int(ranuni(0) * 4) + 1, '09:00't, '12:00't, '15:00't, '18:00't, '21:00't);
    column4 = _n_;
    column5 = _n_;
    column6 = _n_;
    output;
  end;
run;
